import base64
from pyrate.main import Pyrate


class HarvestPyrate(Pyrate):

    # These variables should be set on implementation
    auth_user = ''
    auth_pass = ''
    organisation_name = ''

    http_methods = ['GET', 'POST']
    return_formats = ['json']
    default_body_content = {}
    auth_type = 'BASIC_AUTH'
    connection_check_method = ('GET', 'account/who_am_i')

    def __init__(self):
        self.base_url = 'https://' + self.organisation_name + '.harvestapp.com/'
        self.default_http_method = self.http_methods[0]
        self.default_return_format = self.return_formats[0]
        self.default_header_content = {
            'Authorization': 'Basic ' + base64.b64encode(self.auth_user + ":" + self.auth_pass).rstrip()
        }


from requests_oauthlib import OAuth1

from pyrate.main import Pyrate


class TwitterPyrate(Pyrate):

    base_url = 'https://api.twitter.com/1.1/'
    http_methods = ['GET', 'POST']
    default_http_method = http_methods[0]
    return_formats = ['json']
    default_return_format = return_formats[0]
    default_body_content = {}
    default_header_content = {}
    auth_type = 'OAUTH1'
    connection_check_method = ('GET', 'account/verify_credentials')

    # These variables should be set on implementation
    oauth_consumer_key = ''
    oauth_consumer_secret = ''
    oauth_token = ''
    oauth_token_secret = ''

    def get_oauth(self):
        if self.oauth_token != "" and self.oauth_token_secret != "":
            return OAuth1(self.oauth_consumer_key,
                          client_secret=self.oauth_consumer_secret,
                          resource_owner_key=self.oauth_token,
                          resource_owner_secret=self.oauth_token_secret)
        else:
            raise Exception("Please set your oauth_token and oauth_token_secret first!")



import requests


class Pyrate:

    http_methods = ['GET', 'POST', 'PUT', 'DELETE', 'OPTIONS']
    return_formats = ['json']
    default_header_content = None
    default_body_content = None
    default_http_method = None
    default_return_format = None
    connection_check_method = None
    auth_type = None
    api_key = None
    base_url = None

    def __init__(self):
        self.default_http_method = self.http_methods[0]
        self.default_return_format = self.return_formats[0]

    def get_oauth(self):
        raise NotImplementedError("OAuth methods need to be implemented by subclasses!")

    def check_connection(self):
        return self.do(self.connection_check_method[1], http_method=self.connection_check_method[0])

    def do(self, method, content=None, headers=None, http_method=None, return_format=None):

        request_body = self.default_body_content
        if content is not None:
            request_body.update(content)

        request_headers = self.default_header_content
        if headers is not None:
            request_headers.update(headers)

        if http_method is None:
            http_method = self.default_http_method

        if return_format is None:
            if self.default_return_format:
                return_format = "." + self.default_return_format
            else:
                return_format = ''

        request_url = self.base_url + method + return_format
        request_headers = self.default_header_content
        request_body = self.default_body_content

        return self.do_request(http_method, request_url, request_headers, request_body, return_format)

    def do_request(self, http_method, url, headers, body, return_format):

        if self.auth_type == 'OAUTH1':
            auth_data = self.get_oauth()
        else:
            auth_data = None

        if http_method.upper() == 'GET':
            r = requests.get(url, headers=headers, auth=auth_data)

        elif http_method.upper() == 'POST':
            r = requests.post(url, params=body, headers=headers, auth=auth_data)

        elif http_method.upper() == 'PUT':
            r = requests.put(url, params=body, headers=headers, auth=auth_data)

        elif http_method.upper() == 'DELETE':
            r = requests.delete(url, params=body, headers=headers, auth=auth_data)

        elif http_method.upper() == 'OPTIONS':
            r = requests.options(url, params=body, headers=headers, auth=auth_data)

        return self.handle_response(r, return_format)

    def handle_response(self, response, return_format):
        try:
            return response.json()
        except ValueError:
            return response.content


from pyrate.main import Pyrate

class MailchimpPyrate(Pyrate):

    # This variable should be set on implementation
    api_key = ''

    http_methods = ['POST']
    default_http_method = http_methods[0]
    return_formats = ['JSON', 'XML', 'PHP']
    default_header_content = {}
    auth_type = 'API_KEY'
    connection_check_method = ('POST', 'helper/ping')

    def __init__(self):
        self.default_http_method = self.http_methods[0]
        self.default_return_format = self.return_formats[0]
        self.base_url = 'https://' + self.api_key[-3:] + '.api.mailchimp.com/2.0/'
        self.default_body_content = {
            'apikey': self.api_key
        }




#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
import requests
from requests_oauthlib import OAuth1
from urlparse import parse_qs


def twitter_oauth():
    print
    print
    print
    os.system("clear")
    print "Twitter OAuth"
    print "-------------"
    print
    oauth_consumer_key = raw_input("Please enter your OAuth Consumer Key: ")
    oauth_consumer_secret = raw_input("Please enter your OAuth Consumer Secret: ")

    oauth_token, oauth_token_secret = setup_twitter_oauth(oauth_consumer_key, oauth_consumer_secret)

    print "These are your OAuth tokens. You need them for the TwitterPyrate."
    print "OAuth Token: " + oauth_token
    print "OAuth Token Secret: " + oauth_token_secret

def setup_twitter_oauth(oauth_consumer_key, oauth_consumer_secret):

    oauth_request_token_url = 'https://api.twitter.com/oauth/request_token'
    oauth_authorize_url = 'https://api.twitter.com/oauth/authorize?oauth_token='
    oauth_access_token_url = 'https://api.twitter.com/oauth/access_token'

    """Authorize your app via identifier."""
    # Request token
    oauth = OAuth1(oauth_consumer_key, client_secret=oauth_consumer_secret)
    r = requests.post(url=oauth_request_token_url, auth=oauth)
    credentials = parse_qs(r.content)

    resource_owner_key = credentials.get('oauth_token')[0]
    resource_owner_secret = credentials.get('oauth_token_secret')[0]

    # Authorize
    authorize_url = oauth_authorize_url + resource_owner_key
    print 'Please go here to authorize:'
    print authorize_url

    verifier = raw_input('Please input the verifier: ')
    oauth = OAuth1(oauth_consumer_key,
                   client_secret=oauth_consumer_secret,
                   resource_owner_key=resource_owner_key,
                   resource_owner_secret=resource_owner_secret,
                   verifier=verifier)

    # Finally, Obtain the Access Token
    r = requests.post(url=oauth_access_token_url, auth=oauth)
    credentials = parse_qs(r.content)
    token = credentials.get('oauth_token')[0]
    secret = credentials.get('oauth_token_secret')[0]

    return token, secret

# When were adding more we'll use docopt!

def main():
    os.system("clear")
    print "                         _"
    print "                        | |"
    print "   _ __  _   _ _ __ __ _| |_ ___"
    print "  | '_ \| | | | '__/ _` | __/ _ \ "
    print "  | |_) | |_| | | | (_| | ||  __/"
    print "  | .__/ \__, |_|  \__,_|\__\___|"
    print "  | |     __/ |"
    print "  |_|    |___/"
    print
    print

    print "# These is Pyrate's commandline tool. For the usage of pyrate,"
    print "# please read the docs: https://github.com/Chive/pyrate"
    print ""
    print
    print "# Tasks"
    print "1 Generate Twitter OAuth Tokens"
    print "0 Exit"
    print
    c = raw_input("Your Choice: ")

    if c == '1':
        twitter_oauth()

    elif c == '0':
        exit("Goodbye")

    else:
        exit("Invalid choice. Goodbye")



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from setuptools import setup, find_packages
from pyrate import __version__


CLASSIFIERS = [
    'Development Status :: 3 - Alpha',
    'Environment :: Web Environment',
    'Intended Audience :: Developers',
    'License :: OSI Approved :: MIT License',
    'Operating System :: OS Independent',
    'Programming Language :: Python',
    'Topic :: Communications',
    'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    'Programming Language :: Python',
]

setup(
    name='pyrate',
    version=__version__,
    description='Pyrate is a python wrapper for restful web apis. It\'s like magic but quicker.',
    author='Kim Thoenen',
    author_email='kim@smuzey.ch',
    url='https://github.com/chive/pyrate',
    packages=find_packages(),
    license='MIT',
    platforms=['OS Independent'],
    classifiers=CLASSIFIERS,
    include_package_data=True,
    zip_safe=False,

    install_requires=[
        open("requirements.txt").readlines(),
    ],
    entry_points={
        'console_scripts': [
            'pyrate = pyrate.scripts.cli:main',
        ]
    },)